@@ -23,7 +23,7 @@ struct janus_session { // "login" session
 	struct obj obj;
 	uint64_t id;
 	mutex_t lock;
-	time_t last_act;
+	int64_t last_act;
 	janus_websockets_ht websockets; // controlling transports, websocket_conn -> websocket_conn
 	janus_handles_set handles; // handle ID -> 0x1. handle ID owned by janus_handles
 };
@@ -55,18 +55,17 @@ struct janus_room {
 TYPED_GHASHTABLE(janus_rooms_ht, uint64_t, struct janus_room, int64_hash, int64_eq, NULL, NULL)
 
 
-TYPED_GHASHTABLE(janus_tokens_ht, char, time_t, c_str_hash, c_str_equal, g_free, g_free)
+TYPED_GHASHTABLE(janus_tokens_ht, char, int64_t, c_str_hash, c_str_equal, g_free, g_free)
 
 
-static mutex_t janus_lock;
+static mutex_t janus_lock = MUTEX_STATIC_INIT;
 static janus_tokens_ht janus_tokens; // auth tokens, currently mostly unused
 static janus_sessions_ht janus_sessions; // session ID -> session. holds a session reference
 static janus_handles_ht janus_handles; // handle ID -> handle
 static janus_rooms_ht janus_rooms; // room ID -> room
 
 
-static void __janus_session_free(void *p) {
-	struct janus_session *s = p;
+static void __janus_session_free(struct janus_session *s) {
 	if (t_hash_table_size(s->websockets) != 0)
 		ilog(LOG_WARN, "Janus session is leaking %i WS references", t_hash_table_size(s->websockets));
 	t_hash_table_destroy(s->websockets);
@@ -87,7 +86,7 @@ static struct janus_session *janus_get_session(uint64_t id) {
 	if (!ret)
 		return NULL;
 	mutex_lock(&ret->lock);
-	ret->last_act = rtpe_now.tv_sec;
+	ret->last_act = rtpe_now;
 	mutex_unlock(&ret->lock);
 	return ret;
 }
@@ -121,7 +120,7 @@ static struct call_monologue *janus_get_monologue(uint64_t handle_id, call_t *ca
 {
 	g_autoptr(char) handle_buf = NULL;
 	handle_buf = g_strdup_printf("%" PRIu64, handle_id);
-	str handle_str = STR_INIT(handle_buf);
+	str handle_str = STR(handle_buf);
 
 	return fn(call, &handle_str);
 }
@@ -133,11 +132,9 @@ static void janus_send_json_sync_response(struct websocket_message *wm, JsonBuil
 	char *result = glib_json_print(builder);
 
 	if (wm->method == M_WEBSOCKET)
-		websocket_write_text(wm->wc, result, true);
-	else {
-		websocket_http_response(wm->wc, code, "application/json", strlen(result));
-		websocket_write_http(wm->wc, result, true);
-	}
+		websocket_write_text(wm->wc, result);
+	else
+		websocket_http_complete(wm->wc, code, "application/json", strlen(result), result);
 
 	g_free(result);
 }
@@ -155,7 +152,7 @@ static void janus_send_json_async(struct janus_session *session, JsonBuilder *bu
 	struct websocket_conn *wc;
 	while (t_hash_table_iter_next(&iter, NULL, &wc)) {
 		// lock order constraint: janus_session lock first, websocket_conn lock second
-		websocket_write_text(wc, result, true);
+		websocket_write_text(wc, result);
 	}
 
 	g_free(result);
@@ -194,7 +191,7 @@ static const char *janus_videoroom_create(struct janus_session *session, struct
 		return "User already exists in a room";
 
 	// create new videoroom
-	struct janus_room *room = g_slice_alloc0(sizeof(*room));
+	struct janus_room *room = g_new0(__typeof(*room), 1);
 
 	if (json_reader_read_member(reader, "publishers"))
 		room->num_publishers = jr_str_int(reader);
@@ -227,8 +224,7 @@ static const char *janus_videoroom_create(struct janus_session *session, struct
 		room->id = room_id;
 		if (t_hash_table_lookup(janus_rooms, &room->id))
 			continue;
-		room->call_id.s = janus_call_id(room_id);
-		room->call_id.len = strlen(room->call_id.s);
+		room->call_id = STR(janus_call_id(room_id));
 		call_t *call = call_get_or_create(&room->call_id, true);
 		if (!call) {
 			ilog(LOG_WARN, "Call with reserved Janus ID '" STR_FORMAT
@@ -236,8 +232,8 @@ static const char *janus_videoroom_create(struct janus_session *session, struct
 			g_free(room->call_id.s);
 			continue;
 		}
-		if (!call->created_from)
-			call->created_from = "janus";
+		if (!call->created_from.len)
+			call->created_from = STR_CONST("janus");
 		t_hash_table_insert(janus_rooms, &room->id, room);
 		rwlock_unlock_w(&call->master_lock);
 		obj_put(call);
@@ -315,7 +311,7 @@ static const char *janus_videoroom_destroy(struct janus_session *session,
 	t_hash_table_destroy(room->publishers);
 	t_hash_table_destroy(room->subscribers);
 	t_hash_table_destroy(room->feeds);
-	g_slice_free1(sizeof(*room), room);
+	g_free(room);
 
 	//XXX notify?
 
@@ -335,17 +331,17 @@ static void janus_add_publisher_details(JsonBuilder *builder, struct call_monolo
 	json_builder_set_member_name(builder, "streams");
 	json_builder_begin_array(builder);
 
-	const char *a_codec = NULL, *v_codec = NULL;
+	str *a_codec = NULL, *v_codec = NULL;
 
 	for (unsigned int i = 0; i < ml->medias->len; i++) {
 		struct call_media *media = ml->medias->pdata[i];
 		if (!media)
 			continue;
 
-		const char *codec = NULL;
+		str *codec = NULL;
 		for (__auto_type k = media->codecs.codec_prefs.head; k; k = k->next) {
 			rtp_payload_type *pt = k->data;
-			codec = pt->encoding.s;
+			codec = &pt->encoding;
 			// XXX check codec support?
 			break;
 		}
@@ -353,13 +349,13 @@ static void janus_add_publisher_details(JsonBuilder *builder, struct call_monolo
 		json_builder_begin_object(builder);
 
 		json_builder_set_member_name(builder, "type");
-		json_builder_add_string_value(builder, media->type.s);
+		glib_json_builder_add_str(builder, &media->type);
 		json_builder_set_member_name(builder, "mindex");
 		json_builder_add_int_value(builder, media->index - 1);
 
 		json_builder_set_member_name(builder, "mid");
 		if (media->media_id.s)
-			json_builder_add_string_value(builder, media->media_id.s);
+			glib_json_builder_add_str(builder, &media->media_id);
 		else
 			json_builder_add_null_value(builder);
 
@@ -369,7 +365,7 @@ static void janus_add_publisher_details(JsonBuilder *builder, struct call_monolo
 		}
 		else if (codec) {
 			json_builder_set_member_name(builder, "codec");
-			json_builder_add_string_value(builder, codec);
+			glib_json_builder_add_str(builder, codec);
 
 			if (media->type_id == MT_AUDIO && !a_codec)
 				a_codec = codec;
@@ -384,12 +380,12 @@ static void janus_add_publisher_details(JsonBuilder *builder, struct call_monolo
 
 	if (a_codec) {
 		json_builder_set_member_name(builder, "audio_codec");
-		json_builder_add_string_value(builder, a_codec);
+		glib_json_builder_add_str(builder, a_codec);
 	}
 
 	if (v_codec) {
 		json_builder_set_member_name(builder, "video_codec");
-		json_builder_add_string_value(builder, v_codec);
+		glib_json_builder_add_str(builder, v_codec);
 	}
 
 	// TODO add "display"
@@ -463,7 +459,7 @@ TYPED_GQUEUE(janus_ret_streams, uint64_t);
 static void janus_clear_ret_streams(janus_ret_streams_q *q) {
 	uint64_t *id;
 	while ((id = t_queue_pop_head(q)))
-		g_slice_free1(sizeof(*id), id);
+		g_free(id);
 }
 
 G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC(janus_ret_streams_q, janus_clear_ret_streams);
@@ -607,7 +603,7 @@ static const char *janus_videoroom_join(struct websocket_message *wm, struct jan
 
 					g_string_append_printf(feed_ids, "%" PRIu64 ", ", fid);
 
-					uint64_t *fidp = g_slice_alloc(sizeof(*fidp));
+					uint64_t *fidp = g_new(__typeof(*fidp), 1);
 					*fidp = fid;
 					t_queue_push_tail(&ret_streams, fidp);
 				}
@@ -626,7 +622,7 @@ static const char *janus_videoroom_join(struct websocket_message *wm, struct jan
 				call_get_or_create_monologue);
 
 		g_auto(sdp_ng_flags) flags;
-		call_ng_flags_init(&flags, OP_REQUEST);
+		call_ng_flags_init(&flags, OP_SUBSCRIBE_REQ);
 
 		flags.generate_mid = 1;
 		flags.rtcp_mirror = 1;
@@ -851,7 +847,7 @@ static const char *janus_videoroom_configure(struct websocket_message *wm, struc
 		if (strcmp(jsep_type, "offer"))
 			return "Not an offer";
 
-		g_auto(str) sdp_in = STR_INIT_DUP(jsep_sdp);
+		str sdp_in = call_str_cpy_c(jsep_sdp);
 
 		g_auto(sdp_ng_flags) flags;
 		g_auto(sdp_sessions_q) parsed = TYPED_GQUEUE_INIT;
@@ -956,7 +952,15 @@ static const char *janus_videoroom_start(struct websocket_message *wm, struct ja
 	if (strcmp(jsep_type, "answer"))
 		return "Not an answer";
 
-	g_auto(str) sdp_in = STR_INIT_DUP(jsep_sdp);
+	struct janus_room *room = t_hash_table_lookup(janus_rooms, &room_id);
+	*retcode = 426;
+	if (!room)
+		return "No such room";
+	g_autoptr(call_t) call = call_get(&room->call_id);
+	if (!call)
+		return "No such room";
+
+	str sdp_in = call_str_cpy_c(jsep_sdp);
 
 	g_auto(sdp_ng_flags) flags;
 	g_auto(sdp_sessions_q) parsed = TYPED_GQUEUE_INIT;
@@ -965,16 +969,11 @@ static const char *janus_videoroom_start(struct websocket_message *wm, struct ja
 	*retcode = 512;
 	if (sdp_parse(&sdp_in, &parsed, &flags))
 		return "Failed to parse SDP";
+
+	*retcode = 512;
 	if (sdp_streams(&parsed, &streams, &flags))
 		return "Incomplete SDP specification";
 
-	struct janus_room *room = t_hash_table_lookup(janus_rooms, &room_id);
-	*retcode = 426;
-	if (!room)
-		return "No such room";
-	g_autoptr(call_t) call = call_get(&room->call_id);
-	if (!call)
-		return "No such room";
 	*retcode = 456;
 	uint64_t *feed_id = t_hash_table_lookup(room->subscribers, &handle->id);
 	if (!feed_id)
@@ -1083,7 +1082,7 @@ static const char *janus_videoroom(struct websocket_message *wm, struct janus_se
 	const char *req = json_reader_get_string_value(reader);
 	if (!req)
 		goto err;
-	str req_str = STR_INIT(req);
+	str req_str = STR(req);
 	json_reader_end_member(reader);
 
 	switch (__csh_lookup(&req_str)) {
@@ -1149,8 +1148,8 @@ static const char *janus_add_token(JsonReader *reader, JsonBuilder *builder, boo
 	if (!token)
 		return "JSON object does not contain 'token' key";
 
-	time_t *now = g_malloc(sizeof(*now));
-	*now = rtpe_now.tv_sec;
+	int64_t *now = g_malloc(sizeof(*now));
+	*now = rtpe_now;
 	mutex_lock(&janus_lock);
 	t_hash_table_replace(janus_tokens, g_strdup(token), now);
 	mutex_unlock(&janus_lock);
@@ -1176,10 +1175,10 @@ static const char *janus_create(JsonReader *reader, JsonBuilder *builder, struct
 		session_id = jr_str_int(reader);
 	json_reader_end_member(reader);
 
-	struct janus_session *session = obj_alloc0("janus_session", sizeof(*session), __janus_session_free);
+	__auto_type session = obj_alloc0(struct janus_session, __janus_session_free);
 	mutex_init(&session->lock);
 	mutex_lock(&session->lock); // not really necessary but Coverity complains
-	session->last_act = rtpe_now.tv_sec;
+	session->last_act = rtpe_now;
 	session->websockets = janus_websockets_ht_new();
 	session->handles = janus_handles_set_new();
 
@@ -1279,11 +1278,11 @@ void janus_media_up(struct call_media *media) {
 	json_builder_add_int_value(builder, handle);
 	json_builder_set_member_name(builder, "mid");
 	if (media->media_id.s)
-		json_builder_add_string_value(builder, media->media_id.s);
+		glib_json_builder_add_str(builder, &media->media_id);
 	else
 		json_builder_add_null_value(builder);
 	json_builder_set_member_name(builder, "type");
-	json_builder_add_string_value(builder, media->type.s);
+	glib_json_builder_add_str(builder, &media->type);
 	json_builder_set_member_name(builder, "receiving");
 	json_builder_add_boolean_value(builder, true);
 	json_builder_end_object(builder); // }
@@ -1312,7 +1311,7 @@ static const char *janus_attach(JsonReader *reader, JsonBuilder *builder, struct
 		return "Unsupported plugin";
 	json_reader_end_member(reader);
 
-	struct janus_handle *handle = g_slice_alloc0(sizeof(*handle));
+	struct janus_handle *handle = g_new0(__typeof(*handle), 1);
 	mutex_lock(&janus_lock);
 	handle->session = obj_get(session);
 	uint64_t handle_id = 0;
@@ -1347,7 +1346,7 @@ static void janus_destroy_handle(struct janus_handle *handle) {
 	// destroy handle
 	if (handle->session)
 		obj_put(handle->session);
-	g_slice_free1(sizeof(*handle), handle);
+	g_free(handle);
 
 	if (!room_id)
 		return;
@@ -1539,7 +1538,7 @@ static const char *janus_message(struct websocket_message *wm, JsonReader *reade
 		json_builder_set_member_name(builder, "type");
 		json_builder_add_string_value(builder, jsep_type_out);
 		json_builder_set_member_name(builder, "sdp");
-		json_builder_add_string_value(builder, jsep_sdp_out.s);
+		glib_json_builder_add_str(builder, &jsep_sdp_out);
 		json_builder_end_object(builder); // }
 	}
 
@@ -1614,9 +1613,15 @@ static const char *janus_trickle(JsonReader *reader, struct janus_session *sessi
 		call_id = janus_call_id(handle->room);
 
 		struct janus_room *room = t_hash_table_lookup(janus_rooms, &handle->room);
-
-		if (room)
-			call = call_get(&room->call_id);
+		if (!room) {
+			*retcode = 426;
+			return "No such room";
+		}
+		call = call_get(&room->call_id);
+		if (!call) {
+			*retcode = 426;
+			return "No such room";
+		}
 	}
 
 	// set up "streams" structures to use an trickle ICE update. these must be
@@ -1628,18 +1633,19 @@ static const char *janus_trickle(JsonReader *reader, struct janus_session *sessi
 	// top-level structures first, with auto cleanup
 	g_auto(sdp_streams_q) streams = TYPED_GQUEUE_INIT;
 	g_autoptr(ng_buffer) ngbuf = ng_buffer_new(NULL);
+	bencode_buffer_init(&ngbuf->buffer);
 	g_auto(sdp_ng_flags) flags;
 	call_ng_flags_init(&flags, OP_OTHER);
 
 	// then the contained structures, and add them in
-	struct stream_params *sp = g_slice_alloc0(sizeof(*sp));
+	struct stream_params *sp = g_new0(__typeof(*sp), 1);
 	t_queue_push_tail(&streams, sp);
-	struct ice_candidate *cand = g_slice_alloc0(sizeof(*cand));
+	struct ice_candidate *cand = g_new0(__typeof(*cand), 1);
 	t_queue_push_tail(&sp->ice_candidates, cand);
 
 	// allocate and parse candidate
 	str cand_str;
-	bencode_strdup_str(&ngbuf->buffer, &cand_str, candidate);
+	cand_str = bencode_strdup_str(&ngbuf->buffer, candidate);
 	str_shift_cmp(&cand_str, "candidate:"); // skip prefix
 	if (!cand_str.len) // end of candidates
 		return NULL;
@@ -1656,12 +1662,12 @@ static const char *janus_trickle(JsonReader *reader, struct janus_session *sessi
 
 	g_autoptr(char) handle_buf = NULL;
 	handle_buf = g_strdup_printf("%" PRIu64, handle_id);
-	bencode_strdup_str(&ngbuf->buffer, &flags.from_tag, handle_buf);
-	bencode_strdup_str(&ngbuf->buffer, &flags.call_id, call_id);
+	flags.from_tag = bencode_strdup_str(&ngbuf->buffer, handle_buf);
+	flags.call_id = bencode_strdup_str(&ngbuf->buffer, call_id);
 
 	// populate and allocate a=mid
 	if (sdp_mid)
-		bencode_strdup_str(&ngbuf->buffer, &sp->media_id, sdp_mid);
+		sp->media_id = bencode_strdup_str(&ngbuf->buffer, sdp_mid);
 
 	// check m= line index
 	if (sdp_m_line >= 0)
@@ -1670,7 +1676,7 @@ static const char *janus_trickle(JsonReader *reader, struct janus_session *sessi
 	// ufrag can be given in-line or separately
 	sp->ice_ufrag = cand->ufrag;
 	if (!sp->ice_ufrag.len && ufrag)
-		bencode_strdup_str(&ngbuf->buffer, &sp->ice_ufrag, ufrag);
+		sp->ice_ufrag = bencode_strdup_str(&ngbuf->buffer, ufrag);
 
 	// finally do the update
 	trickle_ice_update(ngbuf, call, &flags, &streams);
@@ -1789,7 +1795,7 @@ static const char *websocket_janus_process_json(struct websocket_message *wm,
 
 	ilog(LOG_DEBUG, "Processing '%s' type Janus message", janus_cmd);
 
-	str janus_cmd_str = STR_INIT(janus_cmd);
+	str janus_cmd_str = STR(janus_cmd);
 
 	err = NULL;
 
@@ -1904,7 +1910,7 @@ const char *websocket_janus_process(struct websocket_message *wm) {
 
 
 const char *websocket_janus_get(struct websocket_message *wm) {
-	str uri = STR_INIT(wm->uri);
+	str uri = STR(wm->uri);
 
 	ilog(LOG_DEBUG, "Processing Janus GET: '%s'", wm->uri);
 
@@ -1937,7 +1943,7 @@ const char *websocket_janus_get(struct websocket_message *wm) {
 
 
 const char *websocket_janus_post(struct websocket_message *wm) {
-	str uri = STR_INIT(wm->uri);
+	str uri = STR(wm->uri);
 
 	ilog(LOG_DEBUG, "Processing Janus POST: '%s'", wm->uri);
 
@@ -1965,7 +1971,6 @@ done:
 
 
 void janus_init(void) {
-	mutex_init(&janus_lock);
 	janus_tokens = janus_tokens_ht_new();
 	janus_sessions = janus_sessions_ht_new();
 	janus_handles = janus_handles_ht_new();
@@ -1973,7 +1978,6 @@ void janus_init(void) {
 	// XXX timer thread to clean up orphaned sessions
 }
 void janus_free(void) {
-	mutex_destroy(&janus_lock);
 	t_hash_table_destroy(janus_tokens);
 	t_hash_table_destroy(janus_sessions);
 	t_hash_table_destroy(janus_handles);
