@@ -19,7 +19,6 @@ struct websocket_conn;
 
 struct websocket_output {
 	GString *str;
-	size_t str_done;
 	enum lws_write_protocol protocol;
 	int http_status;
 	const char *content_type;
@@ -67,7 +66,7 @@ static mutex_t websocket_service_lock = MUTEX_STATIC_INIT;
 
 
 static struct websocket_message *websocket_message_new(struct websocket_conn *wc) {
-	struct websocket_message *wm = g_slice_alloc0(sizeof(*wm));
+	struct websocket_message *wm = g_new0(__typeof(*wm), 1);
 	wm->body = g_string_new("");
 	wm->wc = wc;
 	return wm;
@@ -79,13 +78,13 @@ static void websocket_message_free(struct websocket_message **wm) {
 		g_string_free((*wm)->body, TRUE);
 	if ((*wm)->uri)
 		free((*wm)->uri);
-	g_slice_free1(sizeof(**wm), *wm);
+	g_free(*wm);
 	*wm = NULL;
 }
 
 
 static struct websocket_output *websocket_output_new(void) {
-	struct websocket_output *wo = g_slice_alloc0(sizeof(*wo));
+	struct websocket_output *wo = g_new0(__typeof(*wo), 1);
 	// str remains NULL -> unused output slot
 	return wo;
 }
@@ -93,11 +92,11 @@ static struct websocket_output *websocket_output_new(void) {
 static void websocket_output_free(struct websocket_output *wo) {
 	if (wo->str)
 		g_string_free(wo->str, TRUE);
-	g_slice_free1(sizeof(*wo), wo);
+	g_free(wo);
 }
 
 
-// appends to output buffer without triggering a response - unlocked
+// appends to output buffer without triggering a response - non-locking
 static void __websocket_queue_raw(struct websocket_conn *wc, const char *msg, size_t len) {
 	struct websocket_output *wo = t_queue_peek_tail(&wc->output_q);
 
@@ -105,7 +104,6 @@ static void __websocket_queue_raw(struct websocket_conn *wc, const char *msg, si
 		wo->str = g_string_new("");
 		// allocate pre-buffer
 		g_string_set_size(wo->str, LWS_PRE);
-		wo->str_done = LWS_PRE;
 	}
 
 	if (msg && len)
@@ -113,17 +111,8 @@ static void __websocket_queue_raw(struct websocket_conn *wc, const char *msg, si
 }
 
 
-// appends to output buffer without triggering a response
-void websocket_queue_raw(struct websocket_conn *wc, const char *msg, size_t len) {
-	LOCK(&wc->lock);
-	__websocket_queue_raw(wc, msg, len);
-}
-
-
 // num bytes in output buffer
-size_t websocket_queue_len(struct websocket_conn *wc) {
-	LOCK(&wc->lock);
-
+static size_t __websocket_queue_len(struct websocket_conn *wc) {
 	size_t ret = 0;
 	for (__auto_type l = wc->output_q.head; l; l = l->next) {
 		struct websocket_output *wo = l->data;
@@ -134,19 +123,30 @@ size_t websocket_queue_len(struct websocket_conn *wc) {
 }
 
 
-// adds data to output buffer (can be null) and optionally triggers specified response
-void websocket_write_raw(struct websocket_conn *wc, const char *msg, size_t len,
-		enum lws_write_protocol protocol, bool done)
+// adds data to output buffer (can be null) and optionally triggers specified response - non-locking
+static void __websocket_write_raw(struct websocket_conn *wc, const char *msg, size_t len,
+		enum lws_write_protocol protocol)
 {
-	mutex_lock(&wc->lock);
 	__websocket_queue_raw(wc, msg, len);
 	struct websocket_output *wo = t_queue_peek_tail(&wc->output_q);
 	wo->protocol = protocol;
 	t_queue_push_tail(&wc->output_q, websocket_output_new());
+}
 
-	mutex_unlock(&wc->lock);
+// called after all writes are done - non-locking
+// may intermittently release wc->lock
+static void __websocket_write_done(struct websocket_conn *wc) {
+	while (true) {
+		// If this connection is already closed, don't trigger
+		// a wakeup of the service thread and don't request a
+		// writeable callback. We can't write anyway if the
+		// connection is already closed. Prevents a deadlock
+		// between acquiring the service lock here and the
+		// websocket_conn_cleanup called from within the service
+		// loop.
+		if (!wc->wsi)
+			return;
 
-	if (done) {
 		// Sadly lws_callback_on_writable() doesn't do any internal
 		// locking, therefore we must protect it against a concurrently
 		// running lws_service(), as well as against other threads
@@ -174,40 +174,62 @@ void websocket_write_raw(struct websocket_conn *wc, const char *msg, size_t len,
 		mutex_lock(&websocket_callback_lock);
 		lws_cancel_service(websocket_context);
 
-		mutex_lock(&websocket_service_lock);
+		if (mutex_trylock(&websocket_service_lock)) {
+			// Wrong lock order: service_lock should be the outer lock
+			// and wc->lock should be the inner lock. The connection may
+			// be in the process of being closed (websocket_conn_cleanup).
+			// Release wc->lock and try again.
+			mutex_unlock(&websocket_callback_lock);
+			mutex_unlock(&wc->lock);
+			sched_yield();
+			mutex_lock(&wc->lock);
+			continue;
+		}
 		lws_callback_on_writable(wc->wsi);
 
 		mutex_unlock(&websocket_service_lock);
 		mutex_unlock(&websocket_callback_lock);
+
+		break;
 	}
 }
 
 
-// adds data to output buffer (can be null) and triggers specified response: http or binary websocket
-void websocket_write_http_len(struct websocket_conn *wc, const char *msg, size_t len, bool done) {
-	websocket_write_raw(wc, msg, len, LWS_WRITE_HTTP, done);
+// adds data to output buffer (can be null) and triggers specified response: http or binary websocket - non-locking
+void __websocket_write_http_len(struct websocket_conn *wc, const char *msg, size_t len) {
+	__websocket_write_raw(wc, msg, len, LWS_WRITE_HTTP);
 }
-void websocket_write_http(struct websocket_conn *wc, const char *msg, bool done) {
-	websocket_write_http_len(wc, msg, msg ? strlen(msg) : 0, done);
+void __websocket_write_http(struct websocket_conn *wc, const char *msg) {
+	__websocket_write_http_len(wc, msg, msg ? strlen(msg) : 0);
 }
-void websocket_write_text(struct websocket_conn *wc, const char *msg, bool done) {
-	websocket_write_raw(wc, msg, strlen(msg), LWS_WRITE_TEXT, done);
+void __websocket_write_text(struct websocket_conn *wc, const char *msg) {
+	__websocket_write_raw(wc, msg, strlen(msg), LWS_WRITE_TEXT);
 }
-void websocket_write_binary(struct websocket_conn *wc, const char *msg, size_t len, bool done) {
-	websocket_write_raw(wc, msg, len, LWS_WRITE_BINARY, done);
+void __websocket_write_binary(struct websocket_conn *wc, const char *msg, size_t len) {
+	__websocket_write_raw(wc, msg, len, LWS_WRITE_BINARY);
 }
 
-
-void websocket_write_next(struct websocket_conn *wc) {
-	LOCK(&wc->lock);
-	t_queue_push_tail(&wc->output_q, websocket_output_new());
+// singe shot writes with locking
+void websocket_write_text(struct websocket_conn *wc, const char *msg) {
+	{
+		LOCK(&wc->lock);
+		__websocket_write_text(wc, msg);
+		__websocket_write_done(wc);
+	}
+}
+static void websocket_write_binary(struct websocket_conn *wc, const char *msg, size_t len) {
+	{
+		LOCK(&wc->lock);
+		__websocket_write_binary(wc, msg, len);
+		__websocket_write_done(wc);
+	}
 }
 
 
 static const char *websocket_echo_process(struct websocket_message *wm) {
 	ilogs(http, LOG_DEBUG, "Returning %lu bytes websocket echo from %s", (unsigned long) wm->body->len,
 			endpoint_print_buf(&wm->wc->endpoint));
-	websocket_write_binary(wm->wc, wm->body->str, wm->body->len, true);
+	websocket_write_binary(wm->wc, wm->body->str, wm->body->len);
 	return NULL;
 }
 
@@ -240,7 +262,7 @@ static void websocket_process(void *p, void *up) {
 
 	assert(wm != NULL);
 
-	gettimeofday(&rtpe_now, NULL);
+	rtpe_now = now_us();
 
 	const char *err = wm->func(wm);
 	// this may trigger a cleanup/free in another thread, which will then block until our
@@ -305,9 +327,10 @@ static int websocket_dequeue(struct websocket_conn *wc) {
 	if (!wc)
 		return 0;
 
-	int is_http = 0;
+	bool is_http = false;
 
 	mutex_lock(&wc->lock);
+
 	struct websocket_output *wo;
 	struct lws *wsi = wc->wsi;
 	while ((wo = t_queue_pop_head(&wc->output_q))) {
@@ -326,23 +349,26 @@ static int websocket_dequeue(struct websocket_conn *wc) {
 
 		// allocate post-buffer
 		g_string_set_size(wo->str, wo->str->len + LWS_SEND_BUFFER_POST_PADDING);
-		size_t to_send = wo->str->len - wo->str_done - LWS_SEND_BUFFER_POST_PADDING;
+		size_t to_send = wo->str->len - LWS_PRE - LWS_SEND_BUFFER_POST_PADDING;
+
 		if (to_send) {
 			if (to_send > 10000)
-				ilogs(http, LOG_DEBUG, "Writing %lu bytes to LWS", (unsigned long) to_send);
+				ilogs(http, LOG_DEBUG, "Writing %zu bytes to LWS", to_send);
 			else
 				ilogs(http, LOG_DEBUG, "Writing back to LWS: '%.*s'",
-						(int) to_send, wo->str->str + wo->str_done);
-			size_t ret = lws_write(wsi, (unsigned char *) wo->str->str + wo->str_done,
+						(int) to_send, wo->str->str + LWS_PRE);
+			ssize_t ret = lws_write(wsi, (unsigned char *) wo->str->str + LWS_PRE,
 					to_send, wo->protocol);
-			if (ret != to_send)
-				ilogs(http, LOG_ERR, "Invalid LWS write: %lu != %lu",
-						(unsigned long) ret,
-						(unsigned long) to_send);
-			wo->str_done += ret;
+			if (ret <= 0)
+				ilogs(http, LOG_DEBUG, "Failed LWS write: %zd", ret);
+			else if (ret < to_send) {
+				lws_callback_on_writable(wsi);
+				websocket_output_free(wo);
+				break;
+			}
 
 			if (wo->protocol == LWS_WRITE_HTTP)
-				is_http = 1;
+				is_http = true;
 		}
 
 next:
@@ -352,19 +378,16 @@ next:
 
 	mutex_unlock(&wc->lock);
 
-	int ret = 0;
 	if (is_http)
 		if (lws_http_transaction_completed(wsi) == 1) // may destroy `wc`
-			ret = -1;
+			return -1;
 
-	return ret;
+	return 0;
 }
 
-void websocket_http_response(struct websocket_conn *wc, int status, const char *content_type,
+void __websocket_http_response(struct websocket_conn *wc, int status, const char *content_type,
 		ssize_t content_length)
 {
-	LOCK(&wc->lock);
-
 	struct websocket_output *wo = t_queue_peek_tail(&wc->output_q);
 
 	wo->http_status = status;
@@ -374,13 +397,17 @@ void websocket_http_response(struct websocket_conn *wc, int status, const char *
 void websocket_http_complete(struct websocket_conn *wc, int status, const char *content_type,
 		ssize_t content_length, const char *content)
 {
-	websocket_http_response(wc, status, content_type, content_length);
-	websocket_write_http(wc, content, true);
+	{
+		LOCK(&wc->lock);
+		__websocket_http_response(wc, status, content_type, content_length);
+		__websocket_write_http(wc, content);
+		__websocket_write_done(wc);
+	}
 }
 
 
 static const char *websocket_http_ping(struct websocket_message *wm) {
-	ilogs(http, LOG_DEBUG, "Respoding to GET /ping");
+	ilogs(http, LOG_DEBUG, "Responding to GET /ping");
 	websocket_http_complete(wm->wc, 200, "text/plain", 5, "pong\n");
 	return NULL;
 }
@@ -389,7 +416,7 @@ static const char *websocket_http_ping(struct websocket_message *wm) {
 TYPED_GHASHTABLE(metric_types_ht, char, void, c_str_hash, c_str_equal, NULL, NULL)
 
 static const char *websocket_http_metrics(struct websocket_message *wm) {
-	ilogs(http, LOG_DEBUG, "Respoding to GET /metrics");
+	ilogs(http, LOG_DEBUG, "Responding to GET /metrics");
 
 	g_autoptr(stats_metric_q) metrics = statistics_gather_metrics(NULL);
 	g_autoptr(GString) outp = g_string_new("");
@@ -426,13 +453,15 @@ static const char *websocket_http_metrics(struct websocket_message *wm) {
 
 
 // adds printf string to output buffer without triggering response
-static void websocket_queue_printf(struct cli_writer *cw, const char *fmt, ...) {
+static size_t __websocket_queue_printf(struct cli_writer *cw, const char *fmt, ...) {
 	va_list va;
 	va_start(va, fmt);
 	char *s = g_strdup_vprintf(fmt, va);
+	size_t ret = strlen(s);
 	va_end(va);
-	websocket_queue_raw(cw->ptr, s, strlen(s));
+	__websocket_queue_raw(cw->ptr, s, ret);
 	g_free(s);
+	return ret;
 }
 
 
@@ -440,19 +469,45 @@ static const char *websocket_http_cli(struct websocket_message *wm) {
 	assert(strncmp(wm->uri, "/cli/", 5) == 0);
 	char *uri = wm->uri+5;
 
-	ilogs(http, LOG_DEBUG, "Respoding to GET /cli/%s", uri);
+	ilogs(http, LOG_DEBUG, "Responding to GET /cli/%s", uri);
 
-	str uri_cmd = STR_INIT(uri);
+	str uri_cmd = STR(uri);
 
 	struct cli_writer cw = {
-		.cw_printf = websocket_queue_printf,
+		.cw_printf = __websocket_queue_printf,
 		.ptr = wm->wc,
 	};
-	cli_handle(&uri_cmd, &cw);
 
-	size_t len = websocket_queue_len(wm->wc);
+	{
+		LOCK(&wm->wc->lock);
+		cli_handle(&uri_cmd, &cw);
+		size_t len = __websocket_queue_len(wm->wc);
+		__websocket_http_response(wm->wc, 200, "text/plain", len);
+		__websocket_write_http(wm->wc, NULL);
+		__websocket_write_done(wm->wc);
+	}
+
+	return NULL;
+}
+
+
+static const char *websocket_http_cli_post(struct websocket_message *wm) {
+	ilogs(http, LOG_DEBUG, "Responding to POST /cli");
+
+	struct cli_writer cw = {
+		.cw_printf = __websocket_queue_printf,
+		.ptr = wm->wc,
+	};
+
+	{
+		LOCK(&wm->wc->lock);
+		cli_handle(&STR_LEN(wm->body->str, wm->body->len), &cw);
+		size_t len = __websocket_queue_len(wm->wc);
+		__websocket_http_response(wm->wc, 200, "text/plain", len);
+		__websocket_write_http(wm->wc, NULL);
+		__websocket_write_done(wm->wc);
+	}
 
-	websocket_http_complete(wm->wc, 200, "text/plain", len, NULL);
 	return NULL;
 }
 
@@ -460,15 +515,20 @@ static const char *websocket_http_cli(struct websocket_message *wm) {
 static const char *websocket_cli_process(struct websocket_message *wm) {
 	ilogs(http, LOG_DEBUG, "Processing websocket CLI req '%s'", wm->body->str);
 
-	str uri_cmd = STR_INIT_LEN(wm->body->str, wm->body->len);
+	str uri_cmd = STR_LEN(wm->body->str, wm->body->len);
 
 	struct cli_writer cw = {
-		.cw_printf = websocket_queue_printf,
+		.cw_printf = __websocket_queue_printf,
 		.ptr = wm->wc,
 	};
-	cli_handle(&uri_cmd, &cw);
 
-	websocket_write_binary(wm->wc, NULL, 0, true);
+	{
+		LOCK(&wm->wc->lock);
+		cli_handle(&uri_cmd, &cw);
+		__websocket_write_binary(wm->wc, NULL, 0);
+		__websocket_write_done(wm->wc);
+	}
+
 	return NULL;
 }
 
@@ -477,36 +537,43 @@ static void websocket_ng_send_ws(str *cookie, str *body, const endpoint_t *sin,
 		void *p1)
 {
 	struct websocket_conn *wc = p1;
-	if (cookie) {
-		websocket_queue_raw(wc, cookie->s, cookie->len);
-		websocket_queue_raw(wc, " ", 1);
+	{
+		LOCK(&wc->lock);
+		if (cookie) {
+			__websocket_queue_raw(wc, cookie->s, cookie->len);
+			__websocket_queue_raw(wc, " ", 1);
+		}
+		__websocket_queue_raw(wc, body->s, body->len);
+		__websocket_write_binary(wc, NULL, 0);
+		__websocket_write_done(wc);
 	}
-	websocket_queue_raw(wc, body->s, body->len);
-	websocket_write_binary(wc, NULL, 0, true);
 }
 static void websocket_ng_send_http(str *cookie, str *body, const endpoint_t *sin, const sockaddr_t *from,
 		void *p1)
 {
 	struct websocket_conn *wc = p1;
-	websocket_http_response(wc, 200, "application/x-rtpengine-ng",
-			(cookie ? (cookie->len + 1) : 0) + body->len);
-	if (cookie) {
-		websocket_queue_raw(wc, cookie->s, cookie->len);
-		websocket_queue_raw(wc, " ", 1);
+	{
+		LOCK(&wc->lock);
+		__websocket_http_response(wc, 200, "application/x-rtpengine-ng",
+				(cookie ? (cookie->len + 1) : 0) + body->len);
+		if (cookie) {
+			__websocket_queue_raw(wc, cookie->s, cookie->len);
+			__websocket_queue_raw(wc, " ", 1);
+		}
+		__websocket_queue_raw(wc, body->s, body->len);
+		__websocket_write_http(wc, NULL);
+		__websocket_write_done(wc);
 	}
-	websocket_queue_raw(wc, body->s, body->len);
-	websocket_write_http(wc, NULL, true);
 }
 
-static void __ng_buf_free(void *p) {
-	struct websocket_ng_buf *buf = p;
+static void __ng_buf_free(struct websocket_ng_buf *buf) {
 	g_string_free(buf->body, TRUE);
 }
 
 static const char *websocket_ng_process_generic(struct websocket_message *wm,
 		__typeof__(control_ng_process) cb)
 {
-	struct websocket_ng_buf *buf = obj_alloc0("websocket_ng_buf", sizeof(*buf), __ng_buf_free);
+	__auto_type buf = obj_alloc0(struct websocket_ng_buf, __ng_buf_free);
 
 	endpoint_print(&wm->wc->endpoint, buf->addr, sizeof(buf->addr));
 
@@ -515,7 +582,7 @@ static const char *websocket_ng_process_generic(struct websocket_message *wm,
 	// steal body and initialise
 	buf->body = wm->body;
 	wm->body = g_string_new("");
-	str_init_len(&buf->cmd, buf->body->str, buf->body->len);
+	buf->cmd = STR_LEN(buf->body->str, buf->body->len);
 	buf->endpoint = wm->wc->endpoint;
 
 	cb(&buf->cmd, &buf->endpoint, buf->addr, NULL, websocket_ng_send_ws, wm->wc, &buf->obj);
@@ -533,16 +600,16 @@ static const char *websocket_ng_plain_process(struct websocket_message *wm) {
 static const char *websocket_http_ng_generic(struct websocket_message *wm,
 		__typeof__(control_ng_process) cb)
 {
-	struct websocket_ng_buf *buf = obj_alloc0("websocket_ng_buf", sizeof(*buf), __ng_buf_free);
+	__auto_type buf = obj_alloc0(struct websocket_ng_buf, __ng_buf_free);
 
 	endpoint_print(&wm->wc->endpoint, buf->addr, sizeof(buf->addr));
 
-	ilogs(http, LOG_DEBUG, "Respoding to POST /ng from %s", buf->addr);
+	ilogs(http, LOG_DEBUG, "Responding to POST /ng from %s", buf->addr);
 
 	// steal body and initialise
 	buf->body = wm->body;
 	wm->body = g_string_new("");
-	str_init_len(&buf->cmd, buf->body->str, buf->body->len);
+	buf->cmd = STR_LEN(buf->body->str, buf->body->len);
 	buf->endpoint = wm->wc->endpoint;
 
 	if (cb(&buf->cmd, &buf->endpoint, buf->addr, NULL, websocket_ng_send_http, wm->wc,
@@ -623,6 +690,8 @@ static int websocket_http_post(struct websocket_conn *wc) {
 		wm->content_type = CT_JSON;
 	else if (!strcasecmp(ct, "application/x-rtpengine-ng"))
 		wm->content_type = CT_NG;
+	else if (!strcasecmp(ct, "text/plain"))
+		wm->content_type = CT_TEXT;
 	else
 		ilogs(http, LOG_WARN, "Unsupported content-type '%s'", ct);
 
@@ -631,7 +700,7 @@ static int websocket_http_post(struct websocket_conn *wc) {
 
 
 static const char *websocket_http_options_generic(struct websocket_message *wm) {
-	ilogs(http, LOG_DEBUG, "Respoding to OPTIONS");
+	ilogs(http, LOG_DEBUG, "Responding to OPTIONS");
 	websocket_http_complete(wm->wc, 200, NULL, 0, NULL);
 	return NULL;
 }
@@ -678,6 +747,8 @@ static int websocket_http_body(struct websocket_conn *wc, const char *body, size
 		handler = websocket_janus_process;
 	else if (!strncmp(uri, "/janus/", 7) && wm->method == M_POST && wm->content_type == CT_JSON)
 		handler = websocket_janus_post;
+	else if (!strcmp(uri, "/cli") && wm->method == M_POST && wm->content_type == CT_TEXT)
+		handler = websocket_http_cli_post;
 	else
 		handler = websocket_http_404;
 
@@ -694,6 +765,8 @@ static void websocket_conn_cleanup(struct websocket_conn *wc) {
 
 	// wait until all remaining tasks are finished
 	mutex_lock(&wc->lock);
+
+	wc->wsi = NULL;
 	while (wc->jobs)
 		cond_wait(&wc->cond, &wc->lock);
 
@@ -724,7 +797,7 @@ static void websocket_conn_cleanup(struct websocket_conn *wc) {
 	g_string_free(wc->wm->body, TRUE);
 	if (wc->wm->uri)
 		free(wc->wm->uri);
-	g_slice_free1(sizeof(*wc->wm), wc->wm);
+	g_free(wc->wm);
 	wc->wm = NULL;
 	t_queue_clear_full(&wc->output_q, websocket_output_free);
 	if (wc->uri)
@@ -816,7 +889,7 @@ static int websocket_http(struct lws *wsi, enum lws_callback_reasons reason, voi
 {
 	ilogs(http, LOG_DEBUG, "http-only callback %i %p %p", reason, wsi, user);
 
-	gettimeofday(&rtpe_now, NULL);
+	rtpe_now = now_us();
 
 	switch (reason) {
 		case LWS_CALLBACK_PROTOCOL_INIT:
@@ -888,7 +961,7 @@ static int websocket_protocol(struct lws *wsi, enum lws_callback_reasons reason,
 
 	ilogs(http, LOG_DEBUG, "Websocket protocol '%s' callback %i %p %p", name, reason, wsi, wc);
 
-	gettimeofday(&rtpe_now, NULL);
+	rtpe_now = now_us();
 
 	switch (reason) {
 		case LWS_CALLBACK_PROTOCOL_INIT:
@@ -1040,7 +1113,7 @@ static void websocket_cleanup(void) {
 	while (websocket_vhost_configs.length) {
 		struct lws_context_creation_info *vhost = g_queue_pop_head(&websocket_vhost_configs);
 		g_free((void *) vhost->iface);
-		g_slice_free1(sizeof(*vhost), vhost);
+		g_free(vhost);
 	}
 }
 
@@ -1087,6 +1160,9 @@ int websocket_init(void) {
 			LWS_SERVER_OPTION_FAIL_UPON_UNABLE_TO_BIND |
 #endif
 			0,
+#if LWS_LIBRARY_VERSION_MAJOR >= 3 || (LWS_LIBRARY_VERSION_MAJOR == 2 && LWS_LIBRARY_VERSION_MINOR >= 1)
+		.pt_serv_buf_size = rtpe_config.http_buf_size > 0 ? rtpe_config.http_buf_size * 1024LL : 0,
+#endif
 	};
 	websocket_context = lws_create_context(&wci);
 	err = "Failed to create LWS context";
@@ -1098,7 +1174,7 @@ int websocket_init(void) {
 		ilogs(http, LOG_DEBUG, "Starting HTTP/WS '%s'", ifa);
 		endpoint_t eps[2];
 		err = "Failed to parse address/port";
-		if (endpoint_parse_any_getaddrinfo_alt(&eps[0], &eps[1], ifa))
+		if (!endpoint_parse_any_getaddrinfo_alt(&eps[0], &eps[1], ifa))
 			goto err;
 		addr_any_v6_consolidate(eps, have_lws_ipv6);
 
@@ -1112,13 +1188,16 @@ int websocket_init(void) {
 				ipv6_fail = true;
 				continue;
 			}
-			struct lws_context_creation_info *vhost = g_slice_alloc(sizeof(*vhost));
+			struct lws_context_creation_info *vhost = g_new(__typeof(*vhost), 1);
 			g_queue_push_tail(&websocket_vhost_configs, vhost);
 
 			*vhost = (struct lws_context_creation_info) {
 				.port = ep->port,
 				.iface = g_strdup(sockaddr_print_buf(&ep->address)),
 				.protocols = websocket_protocols,
+#if LWS_LIBRARY_VERSION_MAJOR >= 3 || (LWS_LIBRARY_VERSION_MAJOR == 2 && LWS_LIBRARY_VERSION_MINOR >= 1)
+				.pt_serv_buf_size = wci.pt_serv_buf_size,
+#endif
 			};
 			vhost->vhost_name = vhost->iface;
 			if (ep->address.family->af == AF_INET)
@@ -1145,7 +1224,7 @@ int websocket_init(void) {
 		ilogs(http, LOG_DEBUG, "Starting HTTPS/WSS '%s'", ifa);
 		endpoint_t eps[2];
 		err = "Failed to parse address/port";
-		if (endpoint_parse_any_getaddrinfo_alt(&eps[0], &eps[1], ifa))
+		if (!endpoint_parse_any_getaddrinfo_alt(&eps[0], &eps[1], ifa))
 			goto err;
 		addr_any_v6_consolidate(eps, have_lws_ipv6);
 
@@ -1159,7 +1238,7 @@ int websocket_init(void) {
 				ipv6_fail = true;
 				continue;
 			}
-			struct lws_context_creation_info *vhost = g_slice_alloc(sizeof(*vhost));
+			struct lws_context_creation_info *vhost = g_new(__typeof(*vhost), 1);
 			g_queue_push_tail(&websocket_vhost_configs, vhost);
 
 			*vhost = (struct lws_context_creation_info) {
@@ -1169,6 +1248,9 @@ int websocket_init(void) {
 				.ssl_cert_filepath = rtpe_config.https_cert,
 				.ssl_private_key_filepath = rtpe_config.https_key ? : rtpe_config.https_cert,
 				.options = LWS_SERVER_OPTION_DO_SSL_GLOBAL_INIT,
+#if LWS_LIBRARY_VERSION_MAJOR >= 3 || (LWS_LIBRARY_VERSION_MAJOR == 2 && LWS_LIBRARY_VERSION_MINOR >= 1)
+				.pt_serv_buf_size = wci.pt_serv_buf_size,
+#endif
 				// XXX cipher list, key password
 			};
 			vhost->vhost_name = vhost->iface;
