@@ -4,19 +4,22 @@
 #include "socket.h"
 #include "obj.h"
 #include "helpers.h"
-
+#include "containers.h"
 
 struct obj;
 struct streambuf_callback;
 struct streambuf_stream;
 
+TYPED_GHASHTABLE_PROTO(tcp_streams_ht, struct streambuf_stream, struct streambuf_stream)
+
+
 typedef void (*tcp_listener_callback_t)(struct obj *p, socket_t *sock, char *addr, socket_t *);
 typedef void (*streambuf_callback_t)(struct streambuf_stream *);
 
 struct streambuf_listener {
 	socket_t listener;
 	mutex_t lock;
-	GHashTable *streams;
+	tcp_streams_ht streams;
 };
 struct streambuf_stream {
 	struct obj obj;
@@ -30,8 +33,6 @@ struct streambuf_stream {
 
 };
 
-int tcp_listener_init(socket_t *, const endpoint_t *, tcp_listener_callback_t, struct obj *);
-
 int streambuf_listener_init(struct streambuf_listener *listener, const endpoint_t *ep,
 		streambuf_callback_t newconn_func,
 		streambuf_callback_t newdata_func,
