@@ -250,11 +250,11 @@ static void mix_buffer_src_init_pos(struct mix_buffer *mb, mix_buffer_ssrc_sourc
 
 
 static void mix_buff_src_shift_delay(struct mix_buffer *mb, mix_buffer_ssrc_source *src,
-		const struct timeval *last, const struct timeval *now)
+		const int64_t last, const int64_t now)
 {
 	if (!last || !now)
 		return;
-	long long diff_us = timeval_diff(now, last);
+	int64_t diff_us = now - last;
 	if (diff_us <= 0)
 		return;
 	unsigned int samples = mb->clockrate * diff_us / 1000000;
@@ -265,12 +265,12 @@ static void mix_buff_src_shift_delay(struct mix_buffer *mb, mix_buffer_ssrc_sour
 // takes the difference between two time stamps into account, scaled to the given clock rate,
 // to add an additional write-delay for a newly created source
 bool mix_buffer_write_delay(struct mix_buffer *mb, uint32_t ssrc, const void *buf, unsigned int samples,
-		const struct timeval *last, const struct timeval *now)
+		const int64_t last, const int64_t now)
 {
 	LOCK(&mb->lock);
 
 	bool created;
-	g_autoptr(mix_buffer_ssrc_source) src = get_ssrc_full(ssrc, mb->ssrc_hash, &created);
+	g_autoptr(mix_buffer_ssrc_source) src = get_ssrc_full(ssrc, &mb->ssrc_hash, &created);
 	if (!src)
 		return false;
 	if (created)
@@ -310,7 +310,7 @@ bool mix_buffer_write_delay(struct mix_buffer *mb, uint32_t ssrc, const void *bu
 
 static struct ssrc_entry *mix_buffer_ssrc_new(void *p) {
 	struct mix_buffer *mb = p;
-	mix_buffer_ssrc_source *src = obj_alloc0("mix_buffer_ssrc", sizeof(*src), NULL);
+	mix_buffer_ssrc_source *src = obj_alloc0(mix_buffer_ssrc_source, NULL);
 	mix_buffer_src_init_pos(mb, src);
 	return &src->h;
 }
@@ -340,7 +340,7 @@ bool mix_buffer_init_active(struct mix_buffer *mb, enum AVSampleFormat fmt, unsi
 	mb->delay = delay;
 	mb->active = active;
 
-	mb->ssrc_hash = create_ssrc_hash_full_fast(mix_buffer_ssrc_new, mb);
+	ssrc_hash_full_init(&mb->ssrc_hash, mix_buffer_ssrc_new, mb);
 
 	return true;
 }
@@ -348,7 +348,7 @@ bool mix_buffer_init_active(struct mix_buffer *mb, enum AVSampleFormat fmt, unsi
 
 void mix_buffer_destroy(struct mix_buffer *mb) {
 	g_free(mb->buf.v);
-	free_ssrc_hash(&mb->ssrc_hash);
+	ssrc_hash_destroy(&mb->ssrc_hash);
 	mutex_destroy(&mb->lock);
 }
 
