@@ -51,7 +51,7 @@ static void poller_free_item(struct poller_item_int *ele) {
 struct poller *poller_new(void) {
 	struct poller *p;
 
-	p = g_slice_alloc0(sizeof(*p));
+	p = g_new0(__typeof(*p), 1);
 	mutex_init(&p->lock);
 	p->fd = epoll_create1(0);
 	if (p->fd == -1)
@@ -76,7 +76,7 @@ void poller_free(struct poller **pp) {
 	if (p->fd != -1)
 		close(p->fd);
 	p->fd = -1;
-	g_slice_free1(sizeof(*p), p);
+	g_free(p);
 	*pp = NULL;
 }
 
@@ -90,8 +90,7 @@ static int epoll_events(struct poller_item *it, struct poller_item_int *ii) {
 }
 
 
-static void poller_item_free(void *p) {
-	struct poller_item_int *i = p;
+static void poller_item_free(struct poller_item_int *i) {
 	obj_put_o(i->item.obj);
 }
 
@@ -127,7 +126,7 @@ bool poller_add_item(struct poller *p, struct poller_item *i) {
 	if (i->fd >= p->items->len)
 		g_ptr_array_set_size(p->items, i->fd + 1);
 
-	ip = obj_alloc0("poller_item_int", sizeof(*ip), poller_item_free);
+	ip = obj_alloc0(struct poller_item_int, poller_item_free);
 	memcpy(&ip->item, i, sizeof(*i));
 	obj_hold_o(ip->item.obj); /* new ref in *ip */
 	p->items->pdata[i->fd] = obj_get(ip);
@@ -142,23 +141,24 @@ bool poller_add_item(struct poller *p, struct poller_item *i) {
 
 bool poller_del_item_callback(struct poller *p, int fd, void (*callback)(void *), void *arg) {
 	struct poller_item_int *it;
+	bool ret = false;
 
 	if (!p || fd < 0)
-		return false;
+		goto out;
 
 	{
 
 	LOCK(&p->lock);
 
 	if (!p->items) // can happen during shutdown/free only
-		return false;
+		goto out;
 	if (fd >= p->items->len)
-		return false;
+		goto out;
 	if (!(it = p->items->pdata[fd]))
-		return false;
+		goto out;
 
 	if (epoll_ctl(p->fd, EPOLL_CTL_DEL, fd, NULL))
-		return false;
+		goto out;
 
 	p->items->pdata[fd] = NULL; /* stealing the ref */
 
@@ -166,12 +166,15 @@ bool poller_del_item_callback(struct poller *p, int fd, void (*callback)(void *)
 
 	obj_put(it);
 
+	ret = true;
+
+out:
 	if (callback)
 		callback(arg);
 	else
 		close(fd);
 
-	return true;
+	return ret;
 }
 bool poller_del_item(struct poller *p, int fd) {
 	return poller_del_item_callback(p, fd, NULL, NULL);
@@ -200,7 +203,7 @@ static int poller_poll(struct poller *p, int timeout, struct epoll_event *evs, i
 	if (ret <= 0)
 		goto out;
 
-	gettimeofday(&rtpe_now, NULL);
+	rtpe_now = now_us();
 
 	for (i = 0; i < ret; i++) {
 		ev = &evs[i];
@@ -300,14 +303,6 @@ void poller_error(struct poller *p, void *fdp) {
 	it->blocked = 1;
 }
 
-#ifdef HAVE_LIBURING
-
-static unsigned int __uring_thread_loop_dummy(void) { return 0; }
-
-__thread unsigned int (*uring_thread_loop)(void) = __uring_thread_loop_dummy;
-
-#endif
-
 bool poller_isblocked(struct poller *p, void *fdp) {
 	int fd = GPOINTER_TO_INT(fdp);
 	int ret;
@@ -345,7 +340,7 @@ void poller_loop(void *d) {
 		int ret = poller_poll(p, thread_sleep_time, evs, poller_size);
 		if (ret < 0)
 			usleep(20 * 1000);
-		uring_thread_loop();
+		uring_methods.thread_loop();
 	}
 
 	thread_cleanup_pop(true);
