@@ -16,13 +16,15 @@ use Exporter;
 our @ISA;
 our @EXPORT;
 our $launch_cb;
+our $host;
+our $port;
 
 BEGIN {
 	require Exporter;
 	@ISA = qw(Exporter);
-	our @EXPORT = qw(autotest_start new_call offer answer ft tt cid snd srtp_snd rtp rcv srtp_rcv rcv_no
+	our @EXPORT = qw(autotest_start new_call new_call_nc offer answer ft tt cid snd srtp_snd rtp rcv srtp_rcv rcv_no rcv_maybe
 		srtp_dec escape rtpm rtpmre reverse_tags new_ft new_tt crlf sdp_split rtpe_req offer_answer
-		autotest_init subscribe_request subscribe_answer publish use_json);
+		autotest_init subscribe_request subscribe_answer publish use_json rtpe_raw_req);
 };
 
 
@@ -59,39 +61,46 @@ sub autotest_start {
 		ok $rtpe_pid, 'daemon launched in background';
 	}
 
+	if ($launch_cb) {
+		$launch_cb->();
+	}
+
 	return autotest_init();
 }
 
 sub autotest_init {
 	# keep trying to connect to the control socket while daemon is starting up
 	for (1 .. 300) {
-		$c = NGCP::Rtpengine->new($ENV{RTPENGINE_HOST} // '127.0.0.1', $ENV{RTPENGINE_PORT} // 2223);
-		last if $c->{socket};
+		$c = NGCP::Rtpengine->new($host // $ENV{RTPENGINE_HOST} // '127.0.0.1',
+			$port // $ENV{RTPENGINE_PORT} // 2223);
 		Time::HiRes::usleep(100000); # 100 ms x 300 = 30 sec
+
+		$tag_iter = 0;
+		$tag_suffix = '-' . rand();
+
+		my $ok = 0;
+		eval {
+			my $r = $c->req({command => 'ping'});
+			$ok = $r->{result} eq 'pong';
+		};
+
+		next if not $ok;
+
+		# Setup a global die handler.
+		## no critic (Variables::RequireLocalizedPunctuationVars)
+		$SIG{__DIE__} = sub {
+			terminate(@_);
+		};
+
+		last;
 	}
 
-	1;
-	$c->{socket} or die;
-
-	$tag_iter = 0;
-	$tag_suffix = '-' . rand();
-
-	if ($launch_cb) {
-		$launch_cb->();
-	}
-
-	my $r = $c->req({command => 'ping'});
-	ok $r->{result} eq 'pong', 'ping works, daemon operational';
-
 	return 1;
 }
 
-sub new_call {
+sub new_call_nc {
 	my @ports = @_;
-	for my $s (@sockets) {
-		$s->close();
-	}
-	@sockets = ();
+	my @ret;
 	$cid = $tag_iter++ . "-test-callID" . $tag_suffix;
 	$ft = $tag_iter++ . "-test-fromtag" . $tag_suffix;
 	$tt = $tag_iter++ . "-test-totag" . $tag_suffix;
@@ -101,9 +110,17 @@ sub new_call {
 		my $s = IO::Socket::IP->new(Type => &SOCK_DGRAM, Proto => 'udp',
 				LocalHost => $addr, LocalPort => $port)
 				or die;
-		push(@sockets, $s);
+		push(@ret, $s);
 	}
-	return @sockets;
+	push(@sockets, @ret);
+	return @ret;
+}
+sub new_call {
+	for my $s (@sockets) {
+		$s->close();
+	}
+	@sockets = ();
+	new_call_nc(@_);
 }
 sub crlf {
 	my ($s) = @_;
@@ -114,16 +131,21 @@ sub sdp_split {
 	my ($s) = @_;
 	return split(/--------*\n/, $s);
 }
-sub rtpe_req {
-	my ($cmd, $name, $req) = @_;
-	$req->{command} = $cmd;
-	$req->{'call-id'} = $cid;
+sub rtpe_raw_req {
+	my ($req) = @_;
 	my $resp;
 	eval {
 		alarm(3);
 		$resp = $c->req($req);
 		alarm(0);
 	};
+	return $resp || $@;
+}
+sub rtpe_req {
+	my ($cmd, $name, $req) = @_;
+	$req->{command} = $cmd;
+	$req->{'call-id'} //= $cid;
+	my $resp = rtpe_raw_req($req);
 	terminate("'$cmd' request failed ($@)") if $@;
 	is $resp->{result}, 'ok', "$name - '$cmd' status";
 	return $resp;
@@ -206,7 +228,7 @@ sub rtp {
 sub rcv {
 	my ($sock, $port, $match, $cb, $cb_arg) = @_;
 	my $p = '';
-	local $SIG{ALRM} = sub { exit(-10) };
+	local $SIG{ALRM} = sub { die("recv timed out"); };
 	alarm(1);
 	my $addr = $sock->recv($p, 65535, 0) or die;
 	alarm(0);
@@ -232,11 +254,17 @@ sub rcv {
 		}
 	}
 	if ($port == -1 && @matches) {
-		# this is actually wrong and uses the fake Unix domain socket address.
-		# translation should really be handled by the preloaded .so back to
-		# fake v4/v6 address.
-		$addr =~ /\]:(\d+)/s or die;
-		unshift(@matches, $1);
+		if (sockaddr_family($addr) == AF_INET) {
+			my @addr = unpack_sockaddr_in($addr) or die;
+			unshift(@matches, $addr[0], inet_ntoa($addr[1]));
+		}
+		elsif (sockaddr_family($addr) == AF_INET6) {
+			my @addr = unpack_sockaddr_in6($addr) or die;
+			unshift(@matches, $addr[0], inet_ntop(AF_INET6, $addr[1]));
+		}
+		else {
+			die;
+		}
 	}
 	return @matches;
 }
@@ -247,6 +275,13 @@ sub rcv_no {
 	my $addr = $sock->recv($p, 65535, &MSG_DONTWAIT);
 	ok(! defined $addr, "no packet received");
 }
+sub rcv_maybe {
+	my ($sock) = @_;
+	Time::HiRes::sleep(0.1);
+	my $p = '';
+	my $addr = $sock->recv($p, 65535, &MSG_DONTWAIT);
+	ok(1, "whatever packet received or not");
+}
 sub srtp_rcv {
 	my ($sock, $port, $match, $srtp_ctx) = @_;
 	return rcv($sock, $port, $match, \&srtp_dec, $srtp_ctx);
@@ -307,6 +342,8 @@ sub terminate {
 
 	print "hint: rtpe stdout output is at $rtpe_stdout\n";
 	print "hint: rtpe stderr output is at $rtpe_stderr\n";
+	print "last lines:\n";
+	system(qw(tail -n 40), $rtpe_stderr);
 
 	die "error: $msg\n";
 }
